@@ -363,6 +363,104 @@ class PsmPy:
         minor_pred_rstindx['matched_ID'] = ID_match
         self.matched_ids = minor_pred_rstindx[[self.indx, 'matched_ID']]
 
+    def knn_matched_12n(self, matcher, how_many=1):
+        """
+        knn_matched -- Match data using k-nn algorithm
+        Parameters
+        ----------
+        matcher : str
+           string that will used to match - propensity score or proppensity logit
+        how_many : int
+            integer to indicate how many times you want to pull matches for the minor class
+        Returns
+        matched_ids : pd.DataFrame
+            DataFrame with column with matched ID based on k-NN algorithm
+        """
+        # set matcher to matcher
+        matcher = matcher
+        # allocate treatment and control dfs:
+        predicted_data_control = self.predicted_data[self.predicted_data[self.treatment] == 0]
+        predicted_data_treatment = self.predicted_data[self.predicted_data[self.treatment] == 1]
+        # determine which is the major class:
+        if len(predicted_data_treatment) < len(predicted_data_control):
+            min_pred, major_pred = predicted_data_treatment, predicted_data_control
+            major_pred_rstindx = major_pred.reset_index(drop=True)
+            minor_pred_rstindx = min_pred.reset_index(drop=True)
+        elif len(predicted_data_treatment) > len(predicted_data_control):
+            min_pred, major_pred = predicted_data_control, predicted_data_treatment
+            major_pred_rstindx = major_pred.reset_index(drop=True)
+            minor_pred_rstindx = min_pred.reset_index(drop=True)
+        else:
+            min_pred, major_pred = predicted_data_control, predicted_data_treatment
+            major_pred_rstindx = major_pred.reset_index(drop=True)
+            minor_pred_rstindx = min_pred.reset_index(drop=True)
+        major_pred_rstindx_og = major_pred_rstindx.copy()
+
+        # indexes for match needs to keep all those tagged from all loops so must be instantiated outside the loop
+        self.indexes_for_match = []
+
+        for many in range(how_many):
+            # need to fit KNN with larger class
+            knn = NearestNeighbors(n_neighbors=len(major_pred_rstindx), p=2)
+            knn.fit(major_pred_rstindx[[matcher]].to_numpy())
+            distances, indexes = knn.kneighbors(
+                minor_pred_rstindx[[matcher]], n_neighbors=len(major_pred_rstindx))
+            # self.distances = distances
+            # self.indexes = indexes
+
+            # lists to remove elements that have already matched
+            elements_to_remove = []
+            # loop through both distance and indexes from knn simultaneously:
+            for dist, row in zip(distances[:, :], indexes[:, :]):
+                # convert dist and row_ids to lists:
+                dist = np.ndarray.tolist(dist)
+                row = np.ndarray.tolist(row)
+                # check to see if you need to remove elements:
+                if len(elements_to_remove) > 0:
+                    for element in elements_to_remove:
+                        # check is element is in
+                        if element in row:
+                            row.remove(element)
+                        else:
+                            pass
+                        self.indexes_for_match.append(row[0])
+                        elements_to_remove.append(row[0])
+                # if there are NO elements in elements to remove:
+                else:
+                    self.indexes_for_match.append(row[0])
+                    elements_to_remove.append(row[0])
+
+            # remove all used indexes from this round of KNN and prepare for next with the last set removed:
+            # create df for exclusion on those that have matched
+            major_matched = major_pred_rstindx.take(self.indexes_for_match)
+            # run the exclusion for the next run:
+            major_pred_rstindx = pd.merge(major_pred_rstindx, major_matched, on='matched_ID', how="outer", indicator=True
+                                          ).query('_merge=="left_only"')
+
+        # initiate string for ID that will grab the row in accordance with the indx
+        ID_match = []
+        for idxxx in self.indexes_for_match:
+            match = major_pred_rstindx_og.loc[idxxx, self.indx]
+            ID_match.append(match)
+            ID_match_array = np.asarray(ID_match).reshape(
+                (len(minor_pred_rstindx), many))
+
+        # build dataframe passed back to user
+        major_matched = major_pred_rstindx_og.take(self.indexes_for_match)
+        self.df_matched = pd.concat(
+            [minor_pred_rstindx, major_matched], axis=0, ignore_index=True)
+
+        major_list_cols = []
+        # build df of matched ids passed back to user
+        for many in range(how_many):
+            col_name = 'largerclass_id' + str(many)
+            major_list_cols.append(col_name)
+
+        major_class_matched_ids_df = pd.DataFrame(
+            ID_match_array, columns=major_list_cols)
+        self.matched_ids = pd.concat(
+            [minor_pred_rstindx[[self.indx]], major_class_matched_ids_df], axis=1)
+
     def plot_match(self, matched_entity='propensity_logit', Title='Side by side matched controls', Ylabel='Number of patients', Xlabel='propensity logit', names=['treatment', 'control'], colors=['#E69F00', '#56B4E9'], save=False):
         """
         knn_matched -- Match data using k-nn algorithm
